package com.mcwl.web.controller.communityCenter;


import com.mcwl.common.core.domain.AjaxResult;
import com.mcwl.common.core.page.PageDomain;
import com.mcwl.common.core.page.TableDataInfo;
import com.mcwl.communityCenter.domain.dto.QuestionPageRes;
import com.mcwl.communityCenter.domain.dto.QuestionReplyDto;
import com.mcwl.communityCenter.domain.dto.QuestionRes;
import com.mcwl.communityCenter.domain.vo.QuestionVo;
import com.mcwl.communityCenter.service.QuestionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Objects;

/**
 * 提问
 */
@Api(tags = "提问")
@RestController
@RequestMapping("question")
@RequiredArgsConstructor
public class QuestionController {


    private final QuestionService questionService;

    /**
     * 提问
     */
    @PostMapping("addQuestion")
    @ApiOperation(value = "提问")
    public AjaxResult addQuestion(@RequestBody QuestionRes questionRes) {

        questionService.addQuestion(questionRes);

        return AjaxResult.success();

    }


    /**
     * 获取未回复问题列表
     */
    @GetMapping("listUnReplied")
    public TableDataInfo listUnReplied(QuestionPageRes questionPageRes) {

        return questionService.listUnReplied(questionPageRes);
    }

    /**
     * 获取问题详情
     */
    @GetMapping("detail")
    public AjaxResult getQuestionDetail(@NotNull(message = "id不能为空") Long id) {
        QuestionVo questionVo = questionService.getDetail(id);
        if (Objects.isNull(questionVo)) {
            return AjaxResult.error("获取详情失败");
        }
        return AjaxResult.success(questionVo);
    }

    /**
     * 回复
     */
    @PostMapping("reply")
    public AjaxResult reply(QuestionReplyDto questionReplyDto) {

        questionService.reply(questionReplyDto);

        return AjaxResult.success();

    }


}
