package com.mcwl.communityCenter.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.constant.HttpStatus;
import com.mcwl.common.core.page.PageDomain;
import com.mcwl.common.core.page.TableDataInfo;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.common.utils.ShareCodeUtils;
import com.mcwl.communityCenter.constant.InviteConstant;
import com.mcwl.communityCenter.constant.StatusConstant;
import com.mcwl.communityCenter.domain.Invite;
import com.mcwl.communityCenter.domain.InviteCodeMapping;
import com.mcwl.communityCenter.domain.Publish;
import com.mcwl.communityCenter.domain.UserCommunity;
import com.mcwl.communityCenter.domain.vo.PublishVo;
import com.mcwl.communityCenter.mapper.InviteMapper;
import com.mcwl.communityCenter.mapper.PublishMapper;
import com.mcwl.communityCenter.service.InviteCodeMappingService;
import com.mcwl.communityCenter.service.InviteService;
import com.mcwl.communityCenter.service.PublishService;
import com.mcwl.communityCenter.service.UserCommunityService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Service
@RequiredArgsConstructor
public class InviteServiceImpl extends ServiceImpl<InviteMapper, Invite> implements InviteService {

    private final UserCommunityService userCommunityService;

    private final InviteCodeMappingService inviteCodeMappingService;


    @Override
    public String getInviteCode(Long communityId) {


        // 查询用户的社区
        UserCommunity userCommunity = userCommunityService.lambdaQuery()
                .eq(UserCommunity::getCommunityId, communityId)
                .one();

        if (Objects.isNull(userCommunity)) {
            return null;
        }

        String idCode = ShareCodeUtils.idToCode(SecurityUtils.getUserId());

        String inviteCode = "https://www.yaoqing.com?communityId=" + communityId + "&inviteCode=" + idCode;

        // 插入数据库
        InviteCodeMapping inviteCodeMapping = new InviteCodeMapping();
        inviteCodeMapping.setUserId(SecurityUtils.getUserId());
        inviteCodeMapping.setInviteCode(idCode);
        inviteCodeMapping.setCommunityId(communityId);
        inviteCodeMapping.setStatus(StatusConstant.STATUS_AVAILABLE);
        inviteCodeMappingService.save(inviteCodeMapping);

        return inviteCode;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean acceptInvite(Long communityId, String inviteCode) {

        // 解析邀请码
        Long userId = ShareCodeUtils.codeToId(inviteCode);
        if (Objects.isNull(userId)) {
            return false;
        }

        // 判断是否是同一个人
        if (Objects.equals(userId, SecurityUtils.getUserId())) {
            return false;
        }

        // 查询邀请码
        InviteCodeMapping inviteCodeMapping = inviteCodeMappingService.lambdaQuery()
                .eq(InviteCodeMapping::getUserId, userId)
                .eq(InviteCodeMapping::getInviteCode, inviteCode)
                .eq(InviteCodeMapping::getStatus, StatusConstant.STATUS_AVAILABLE)
                .one();

        if (Objects.isNull(inviteCodeMapping)) {
            return false;
        }

        inviteCodeMapping.setStatus(StatusConstant.STATUS_UNAVAILABLE);
        inviteCodeMappingService.updateById(inviteCodeMapping);

        Invite invite = new Invite();
        invite.setTenantId(userId);
        invite.setInviteeUserId(SecurityUtils.getUserId());
        invite.setType(InviteConstant.INVITE_ADMIN);
        invite.setFeeType(InviteConstant.INVITE_FEE);
        baseMapper.insert(invite);
        return true;
    }
}
