package com.mcwl.communityCenter.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.communityCenter.domain.PublicModelCommentVo;
import com.mcwl.communityCenter.domain.PublishCommission;
import com.mcwl.communityCenter.domain.PublishCommissionLike;
import com.mcwl.communityCenter.mapper.PublishCommissionLikeMapper;
import com.mcwl.communityCenter.mapper.PublishCommissionMapper;
import com.mcwl.communityCenter.service.PublishCommissionService;
import com.mcwl.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**评论区业务层
 * @Author：ChenYan
 * @Project：mcwl-ai
 * @Package：com.mcwl.communityCenter.service.impl
 * @Filename：PublishCommissionServiceImpl
 * @Description TODO
 * @Date：2025/1/17 14:28
 */
@Service
public class PublishCommissionServiceImpl extends ServiceImpl<PublishCommissionMapper, PublishCommission> implements PublishCommissionService {
    @Autowired
    private PublishCommissionMapper publishCommentMapper;
    @Autowired
    private PublishCommissionLikeMapper likeMapper;

    @Autowired
    private ISysUserService sysUserService;

    /**
     * 点赞或取消点赞评论
     * @param userId 用户ID
     * @param commentId 评论ID
     * @return 更新后的点赞数
     */
    @Override
    @Transactional
    public int toggleLike(Long userId, Long commentId) {
        // 检查评论是否存在
        PublishCommission comment = this.getById(commentId);
        if (comment == null) {
            throw new RuntimeException("Comment not found");
        }

        // 检查用户是否已经点赞
        PublishCommissionLike like = likeMapper.selectOne(
                new LambdaQueryWrapper<PublishCommissionLike>()
                        .eq(PublishCommissionLike::getUserId, userId)
                        .eq(PublishCommissionLike::getPublishCommissionId, commentId));

        if (like == null) { // 如果没有找到点赞记录，则进行点赞
            like = new PublishCommissionLike();
            like.setUserId(userId);
            like.setCommunityId(1L); // 假设有一个默认的社区ID
            like.setPublishCommissionId(commentId);
            likeMapper.insert(like);

            // 更新评论的点赞数
            comment.setLikeNum(comment.getLikeNum() == null ? 1 : comment.getLikeNum() + 1);
        } else { // 如果找到了点赞记录，则取消点赞
            likeMapper.deleteById(like.getId());

            // 更新评论的点赞数
            comment.setLikeNum(Math.max(0, comment.getLikeNum() - 1));
        }

        // 保存更新后的评论信息
        this.updateById(comment);

        return comment.getLikeNum();
    }

    @Override
    public List<PublicModelCommentVo> getComment(PublishCommission publishCommission) {
        List<PublicModelCommentVo> CommentVoList = new ArrayList<>();

        // 查询所有父评论
        LambdaQueryWrapper<PublishCommission> lqw = new LambdaQueryWrapper<>();
        lqw.eq(PublishCommission::getTenantId, publishCommission.getTenantId())
                .isNull(PublishCommission::getParentId)
                .orderByDesc(PublishCommission::getCreateTime);
        // 添加父评论
        List<PublishCommission> modelCommentList = publishCommentMapper.selectList(lqw);
        for (PublishCommission Comment : modelCommentList) {
            PublicModelCommentVo CommentVo = getModelCommentVo(Comment);
            CommentVoList.add(CommentVo);
        }

        return CommentVoList;
    }


    /**
     * 构建CommentVo对象
     *
     * @param Comment 父评论对象
     * @return CommentVo对象
     */
    @NotNull
    private PublicModelCommentVo getModelCommentVo(PublishCommission Comment) {
        Long userId = Comment.getUserId();
        SysUser sysUser = sysUserService.selectUserById(userId);

        // 构建CommentVo对象
        PublicModelCommentVo CommentVo = new PublicModelCommentVo();
        CommentVo.setUserId(userId);
        CommentVo.setUserName(sysUser.getUserName());
        CommentVo.setUserAvatar(sysUser.getAvatar());
        CommentVo.setCommentId(Comment.getId());
        CommentVo.setContent(Comment.getContent());
        // 查询子评论
        CommentVo.setContentList(getContentList(Comment.getId()));
        CommentVo.setLikeNum(Comment.getLikeNum());
        CommentVo.setCreateTime(Comment.getCreateTime());
        return CommentVo;
    }


    /**
     * 递归查询子评论
     *
     * @param CommentId 父评论id
     * @return List<ModelCommentVo>
     */
    private List<PublicModelCommentVo> getContentList(Long CommentId) {
        List<PublicModelCommentVo> CommentVoList = new ArrayList<>();
        if (Objects.isNull(CommentId)) {
            return CommentVoList;
        }

        // 查询子评论
        LambdaQueryWrapper<PublishCommission> lqw = new LambdaQueryWrapper<PublishCommission>()
                .eq(PublishCommission::getParentId, CommentId)
                .orderByDesc(PublishCommission::getCreateTime);

        List<PublishCommission> CommentList = publishCommentMapper.selectList(lqw);

        for (PublishCommission Comment : CommentList) {
            Long userId = Comment.getUserId();
            SysUser sysUser = sysUserService.selectUserById(userId);
            PublicModelCommentVo CommentVo = new PublicModelCommentVo();
            CommentVo.setUserId(userId);
            CommentVo.setUserName(sysUser.getUserName());
            CommentVo.setUserAvatar(sysUser.getAvatar());
            CommentVo.setCommentId(Comment.getId());
            CommentVo.setContent(Comment.getContent());
            CommentVo.setLikeNum(Comment.getLikeNum());
            CommentVo.setCreateTime(Comment.getCreateTime());


            CommentVoList.add(CommentVo);
        }
        return CommentVoList;
    }
}
